@@ -1,48 +1,22 @@
 # encoding: utf8
-from __future__ import unicode_literals
-import logging
+import tkinter as tk
 import itertools
 import json
-from collections import namedtuple, defaultdict
-
-try:
-    import tkinter as tk
-except:
-    import Tkinter as tk
+import logging
+from collections import defaultdict, namedtuple
 
 __all__ = [
     'BuilderObject', 'EntryBaseBO', 'PanedWindowBO',
     'PanedWindowPaneBO', 'WidgetDescription', 'CLASS_MAP', 'CB_TYPES',
-    'CUSTOM_PROPERTIES', 'register_widget', 'register_property']
+    'CUSTOM_PROPERTIES', 'register_widget', 'register_property',
+    'register_custom_property'
+]
 
 logger = logging.getLogger(__name__)
 
 #
 # Utility functions
 #
-zip_longest = getattr(itertools, 'zip_longest', None)
-if zip_longest is None:
-    def zip_longest(*args, **kw):
-        # zip_longest('ABCD', 'xy', fillvalue='-') --> Ax By C- D-
-        fillvalue = kw.get('fillvalue', None)
-        iterators = [iter(it) for it in args]
-        num_active = len(iterators)
-        if not num_active:
-            return
-        while True:
-            values = []
-            for i, it in enumerate(iterators):
-                try:
-                    value = next(it)
-                except StopIteration:
-                    num_active -= 1
-                    if not num_active:
-                        return
-                    iterators[i] = itertools.repeat(fillvalue)
-                    value = fillvalue
-                values.append(value)
-            yield tuple(values)
-    itertools.zip_longest = zip_longest
 
 
 def grouper(iterable, n, fillvalue=None):
@@ -52,18 +26,12 @@ def grouper(iterable, n, fillvalue=None):
     return itertools.zip_longest(*args, fillvalue=fillvalue)
 
 
-# Python 2 issue:
-try:
-    isinstance(basestring, type)
-except:
-    basestring = str
-
 #
 # BuilderObject
 #
 
 WidgetDescription = namedtuple('WidgetDescription',
-                              ['classname', 'builder', 'label', 'tags'])
+                               ['classname', 'builder', 'label', 'tags'])
 
 CLASS_MAP = {}
 
@@ -89,6 +57,29 @@ def register_property(name, description):
         logger.debug('Registered property %s', name)
 
 
+def register_custom_property(
+        builder_uid, prop_name,
+        editor, default_value=None, help=None,
+        **editor_params):
+    '''Helper function to register a custom property.
+    All custom properties are created using internal dynamic editor.
+    '''
+    description = {
+        'editor': 'dynamic',
+        builder_uid: {
+            'params': {
+                'mode': editor,
+            }
+        }
+    }
+    description[builder_uid]['params'].update(editor_params)
+    if default_value is not None:
+        description[builder_uid]['default'] = default_value
+    if help is not None:
+        description[builder_uid]['help'] = help
+    register_property(prop_name, description)
+
+
 class CB_TYPES:
     '''Callback types'''
     SIMPLE = 'simple'
@@ -102,6 +93,8 @@ class CB_TYPES:
 #
 # Base class
 #
+
+
 class BuilderObject(object):
     """Base class for Widgets created with Builder"""
 
@@ -110,6 +103,7 @@ class BuilderObject(object):
     OPTIONS_CUSTOM = tuple()
     class_ = None
     container = False
+    container_layout = False
     allowed_parents = None
     allowed_children = None
     maxchildren = None
@@ -129,6 +123,20 @@ class BuilderObject(object):
         wdata.layout_required = clsobj.layout_required
         return clsobj
 
+    @classmethod
+    def add_allowed_parent(cls, builder_uid):
+        if cls.allowed_parents is None:
+            cls.allowed_parents = (builder_uid,)
+        else:
+            cls.allowed_parents = cls.allowed_parents + (builder_uid,)
+
+    @classmethod
+    def add_allowed_child(cls, builder_uid):
+        if cls.allowed_children is None:
+            cls.allowed_children = (builder_uid,)
+        else:
+            cls.allowed_children = cls.allowed_children + (builder_uid,)
+
     def __init__(self, builder, wmeta):
         super(BuilderObject, self).__init__()
         self.widget = None
@@ -158,80 +166,78 @@ class BuilderObject(object):
             target = self.widget
         for pname, value in self.wmeta.properties.items():
             if (pname not in self.ro_properties and
-                pname not in self.command_properties):
-                pvalue = self._process_property_value(pname, value)
-                self._set_property(target, pname, pvalue)
-                
+                    pname not in self.command_properties):
+                self._set_property(target, pname, value)
+
     def _process_property_value(self, pname, value):
-        return value
+        propvalue = value
+        if pname in self.tkvar_properties:
+            propvalue = self.builder.create_variable(value)
+            if 'text' in self.wmeta.properties and pname == 'textvariable':
+                propvalue.set(self.wmeta.properties['text'])
+            elif 'value' in self.wmeta.properties and pname == 'variable':
+                propvalue.set(self.wmeta.properties['value'])
+        elif pname in self.tkimage_properties:
+            propvalue = self.builder.get_image(value)
+        elif pname == 'takefocus':
+            if value:
+                propvalue = tk.getboolean(value)
+        return propvalue
 
     def _set_property(self, target_widget, pname, value):
         if pname not in self.__class__.properties:
             msg = "Attempt to set an unknown property '%s' on class '%s'"
             logger.warning(msg, pname, repr(self.class_))
         else:
-            propvalue = value
-            if pname in self.tkvar_properties:
-                propvalue = self.builder.create_variable(value)
-                if 'text' in self.wmeta.properties and pname == 'textvariable':
-                    propvalue.set(self.wmeta.properties['text'])
-                elif 'value' in self.wmeta.properties and pname == 'variable':
-                    propvalue.set(self.wmeta.properties['value'])
-            elif pname in self.tkimage_properties:
-                propvalue = self.builder.get_image(value)
-            elif pname == 'takefocus':
-                if value:
-                    propvalue = tk.getboolean(value)
-
+            propvalue = self._process_property_value(pname, value)
             try:
                 logger.debug('setting property %s = %s', pname, propvalue)
                 target_widget[pname] = propvalue
             except tk.TclError as e:
-                msg = "Failed to set property '{0}' on class '{1}'. TclError:"
-                logger.error(msg, pname, repr(self.class_))
-                logger.exception(e)
+                msg = "Failed to set property '%s' on class '%s'. TclError: %s"
+                logger.error(msg, pname, repr(self.class_), str(e))
+                # logger.exception(e)
 
-    def layout(self, target=None, configure_gridrc=True):
+    def layout(self, target=None):
+        if target is None:
+            target = self.widget
         if self.layout_required:
-            if target is None:
-                target = self.widget
-    
             # Check manager
             manager = self.wmeta.manager
-            logger.debug('Applying %s layout to %s', manager, self.wmeta.identifier)
+            logger.debug(
+                'Applying %s layout to %s',
+                manager,
+                self.wmeta.identifier)
+            properties = self.wmeta.layout_properties
             if manager == 'grid':
-                self._grid_layout(target)
+                target.grid(**properties)
             elif manager == 'pack':
-                self._pack_layout(target)
+                target.pack(**properties)
             elif manager == 'place':
-                self._place_layout(target)
+                target.place(**self.wmeta.layout_properties)
             else:
                 msg = 'Invalid layout manager: {0}'.format(manager)
                 raise Exception(msg)
-        if configure_gridrc:
-            logger.debug('Configurying grid-rc')
-            parent = target.nametowidget(target.winfo_parent())
-            self._gridrc_config(parent)
 
-    def _pack_layout(self, target):
-        properties = dict(self.wmeta.layout_properties)
-        propagate = properties.pop('propagate', 'true')
-        # Do pack
-        target.pack(**properties)
-        if propagate.lower() != 'true':
-            target.pack_propagate(0)
-    
-    def _place_layout(self, target):
-        # Do place
-        target.place(**self.wmeta.layout_properties)
-    
-    def _grid_layout(self, target):
-        properties = dict(self.wmeta.layout_properties)
-        propagate = properties.pop('propagate', 'true')
-        propagate = propagate.lower()
-        target.grid(**properties)
-        if propagate != 'true':
-            target.grid_propagate(0)
+        # container layout
+        if self.container_layout:
+            properties = self.wmeta.container_properties
+            propagate = properties.get('propagate', 'true')
+            propagate = tk.getboolean(propagate)
+            anchor = properties.get('anchor', None)
+
+            container_manager = self.wmeta.container_manager
+            if container_manager == 'grid':
+                if anchor:
+                    target.grid_anchor(anchor)
+                if not propagate:
+                    target.grid_propagate(0)
+                self._gridrc_config(target)
+            elif container_manager == 'pack':
+                if not propagate:
+                    target.pack_propagate(0)
+            elif container_manager is None:
+                raise Exception('Container Manager is none :(')
 
     def _gridrc_config(self, target):
         # configure grid row/col properties:
@@ -276,11 +282,11 @@ class BuilderObject(object):
                 cmd = json.loads(cmd)
                 cmd_name = (cmd['value']).strip()
                 if cmd_name:
-                    commands[cmd_pname]= cmd
+                    commands[cmd_pname] = cmd
                 else:
                     msg = "%s: invalid value for property '%s'."
                     logger.warning(msg, self.wmeta.identifier, cmd_pname)
-        
+
         if isinstance(cmd_bag, dict):
             for cmd_pname, cmd in commands.items():
                 cmd_name = cmd['value']
@@ -330,15 +336,32 @@ class BuilderObject(object):
     #
     # Code generation methods
     #
+    @staticmethod
+    def code_escape_str(value):
+        rval = repr(value)[1:-1]
+        rval.replace('"', '\\"')
+        rval.replace("'", "\\'")
+        return rval
+
+    def _code_get_init_args(self, code_identifier):
+        """Creates dict with properties marked as readonly"""
+        args = {}
+        for rop in self.ro_properties:
+            if rop in self.wmeta.properties:
+                pvalue = self._code_process_property_value(
+                    code_identifier, rop, self.wmeta.properties[rop])
+                args[rop] = pvalue
+        return args
+
     def code_realize(self, boparent, code_identifier=None):
         if code_identifier is not None:
             self._code_identifier = code_identifier
         lines = []
         master = boparent.code_child_master()
-        init_args = self._get_init_args()
+        init_args = self._code_get_init_args(self.code_identifier())
         bag = []
         for pname, value in init_args.items():
-            s = "{0}='{1}'".format(pname, value)
+            s = "{0}={1}".format(pname, value)
             bag.append(s)
         kwargs = ''
         if bag:
@@ -347,18 +370,18 @@ class BuilderObject(object):
                                        self._code_class_name(), master, kwargs)
         lines.append(s)
         return lines
-    
+
     def code_identifier(self):
         if self._code_identifier is None:
             return self.wmeta.identifier
         return self._code_identifier
-    
+
     def code_child_master(self):
         return self.code_identifier()
-    
+
     def code_child_add(self, childid):
         return []
-    
+
     def code_configure(self, targetid=None):
         if targetid is None:
             targetid = self.code_identifier()
@@ -377,33 +400,42 @@ class BuilderObject(object):
             lines.append(line)
         for pname in complex_properties:
             lines.extend(code_bag[pname])
-        
+
         return lines
-    
-    def code_layout(self, targetid=None):
+
+    def code_layout(self, targetid=None, parentid=None):
         if targetid is None:
             targetid = self.code_identifier()
+        if parentid is None:
+            parentid = targetid
+        lines = []
         if self.layout_required:
-            lines = []
             layout_stmt = "{0}.{1}({2})"
             arg_stmt = "{0}='{1}'"
             layout = self.wmeta.layout_properties
             args_bag = []
             for p, v in sorted(layout.items()):
-                if p not in ('propagate', ):
-                    args_bag.append(arg_stmt.format(p, v))
+                args_bag.append(arg_stmt.format(p, v))
             args = ', '.join(args_bag)
-            
+
             manager = self.wmeta.manager
             line = layout_stmt.format(targetid, manager, args)
             lines.append(line)
-            
-            pvalue = str(layout.get('propagate', '')).lower()
-            if 'propagate' in layout and  pvalue == 'false':
+
+        if self.container_layout:
+            manager = self.wmeta.container_manager
+            container_prop = self.wmeta.container_properties
+            pvalue = str(container_prop.get('propagate', '')).lower()
+            if 'propagate' in container_prop and pvalue == 'false':
                 line = '{0}.{1}_propagate({2})'
                 line = line.format(targetid, manager, 0)
                 lines.append(line)
-            
+            if manager == 'grid' and 'anchor' in container_prop:
+                value = container_prop.get('anchor')
+                line = "{0}.{1}_anchor('{2}')"
+                line = line.format(targetid, manager, value)
+                lines.append(line)
+
             lrow_stmt = "{0}.rowconfigure('{1}', {2})"
             lcol_stmt = "{0}.columnconfigure('{1}', {2})"
             rowbag = defaultdict(list)
@@ -422,10 +454,8 @@ class BuilderObject(object):
                 args = ', '.join(bag)
                 line = lcol_stmt.format(targetid, k, args)
                 lines.append(line)
-            return lines
-        else:
-            return []
-    
+        return lines
+
     def _code_class_name(self):
         cname = None
         # try here ?
@@ -438,77 +468,78 @@ class BuilderObject(object):
             else:
                 cname = 'ClassNameNotDefined'
         return cname
-    
+
     def _code_process_properties(self, properties, targetid):
         code_bag = {}
         for pname, value in properties.items():
             if (pname not in self.ro_properties and
-                pname not in self.command_properties):
+                    pname not in self.command_properties):
                 self._code_set_property(targetid, pname, value, code_bag)
-        
+
         # properties
         # determine kw properties or complex properties
         kwproperties = []
         complex_properties = []
         for pname, value in code_bag.items():
-            if isinstance(value, str) or isinstance(value, basestring):
+            if isinstance(value, str) or isinstance(value, str):
                 kwproperties.append(pname)
             else:
                 complex_properties.append(pname)
-        
+
         return (code_bag, kwproperties, complex_properties)
-    
+
+    def _code_process_property_value(self, targetid, pname, value):
+        propvalue = "'{}'".format(value)
+        if pname in self.tkvar_properties:
+            propvalue = self._code_set_tkvariable_property(pname, value)
+        elif pname in self.command_properties:
+            cmd = json.loads(value)
+            propvalue = self._code_define_callback(pname, cmd)
+        elif pname in self.tkimage_properties:
+            propvalue = self.builder.code_create_image(value)
+        elif pname == 'takefocus':
+            propvalue = str(tk.getboolean(value))
+        elif pname == 'text':
+            propvalue = self.code_escape_str(propvalue)
+        return propvalue
+
     def _code_set_property(self, targetid, pname, value, code_bag):
-        if pname in self.OPTIONS_CUSTOM:
-            line = "# TODO - {0}: code for custom option '{1}' not implemented."
-            line = line.format(targetid, pname)
-            code_bag[pname] = [line]
-        else:
-            propvalue = "'{}'".format(value)
-            if pname in self.tkvar_properties:
-                varvalue = None
-                if 'text' in self.wmeta.properties and pname == 'textvariable':
-                    varvalue = self.wmeta.properties['text']
-                elif 'value' in self.wmeta.properties and pname == 'variable':
-                    varvalue = self.wmeta.properties['value']
-                propvalue = self.builder.code_create_variable(value, varvalue)
-            elif pname in self.command_properties:
-                cmd_name = value.strip()
-                callback = self.builder.code_create_callback(
-                    targetid, cmd_name, CB_TYPES.SIMPLE)
-                propvalue = callback
-            elif pname in self.tkimage_properties:
-                propvalue = self.builder.code_create_image(value)
-            elif pname == 'takefocus':
-                propvalue = str(tk.getboolean(value))
-            code_bag[pname] = propvalue
-    
+        code_bag[pname] = self._code_process_property_value(
+            targetid, pname, value)
+
+    def _code_set_tkvariable_property(self, pname, value):
+        '''Create code for tk variable property.
+        Can be used from subclases for custom tk variable properties.'''
+        varvalue = None
+        if 'text' in self.wmeta.properties and pname == 'textvariable':
+            varvalue = self.code_escape_str(self.wmeta.properties['text'])
+        elif 'value' in self.wmeta.properties and pname == 'variable':
+            varvalue = self.code_escape_str(self.wmeta.properties['value'])
+        propvalue = self.builder.code_create_variable(value, varvalue)
+        return propvalue
+
     def code_connect_commands(self):
         commands = {}
         for cmd_pname in self.command_properties:
             cmd = self.wmeta.properties.get(cmd_pname, None)
             if cmd is not None:
-                #print('on_code_connect_commands', cmd)
                 cmd = json.loads(cmd)
                 cmd_name = (cmd['value']).strip()
                 if cmd_name:
-                    commands[cmd_pname]= cmd
+                    commands[cmd_pname] = cmd
                 else:
                     msg = "%s: invalid callback name for property '%s'."
                     logger.warning(msg, self.wmeta.identifier, cmd_pname)
         lines = []
         for cmd_pname, cmd in commands.items():
             callback = self._code_define_callback(cmd_pname, cmd)
-            cmd_code = self._code_connect_command(cmd_pname, callback)
+            cmd_code = self._code_connect_command(cmd_pname, cmd, callback)
             if cmd_code:
                 lines.extend(cmd_code)
         return lines
-    
-    def _code_define_callback(self, cmd_pname, cmd):
-        cmdname = cmd['value']
+
+    def _code_define_callback_args(self, cmd_pname, cmd):
         cmdtype = cmd['cbtype']
-        #print('_code_define_callback', cmd)
-        wid = self.code_identifier()
         args = None
         if cmdtype == CB_TYPES.WITH_WID:
             args = ('widget_id', )
@@ -518,13 +549,56 @@ class BuilderObject(object):
             args = ('first', 'last')
         if cmdtype == CB_TYPES.SCROLL:
             args = ('mode', 'value', 'units')
+        if cmdtype == CB_TYPES.ENTRY_VALIDATE:
+            cb_args = cmd['args']
+            if cb_args:
+                name_map = {
+                    '%d': 'd_action',
+                    '%i': 'i_index',
+                    '%P': 'p_entry_value',
+                    '%s': 's_prev_value',
+                    '%S': 's_new_value',
+                    '%v': 'v_validate',
+                    '%V': 'v_condition',
+                    '%W': 'w_entry_name'
+                }
+                args = []
+                for key in cb_args.split():
+                    args.append(name_map[key])
+
+        return args
+
+    def _code_define_callback(self, cmd_pname, cmd):
+        cmdname = cmd['value']
+        cmdtype = cmd['cbtype']
+        args = self._code_define_callback_args(cmd_pname, cmd)
+        wid = self.code_identifier()
         return self.builder.code_create_callback(wid, cmdname, cmdtype, args)
-    
-    def _code_connect_command(self, cmd, cbname):
+
+    def _code_connect_command(self, cmd_pname, cmd, cbname):
         target = self.code_identifier()
-        line = '{0}.configure({1}={2})'.format(target, cmd, cbname)
-        return (line, )
-    
+        args = self._code_define_callback_args(cmd_pname, cmd)
+        lines = []
+        cmdtype = cmd['cbtype']
+        if args is not None:
+            if cmdtype == CB_TYPES.WITH_WID:
+                wid = self.wmeta.identifier
+                fdef = "_wcmd = lambda wid='{0}': {1}(wid)".format(
+                    wid, cbname)
+                cbname = '_wcmd'
+                lines.append(fdef)
+            if cmdtype == CB_TYPES.ENTRY_VALIDATE:
+                original_cb = cbname
+                tk_args = ["'%{0}'".format(a) for a in args]
+                tk_args = ','.join(tk_args)
+                fdef = "_validatecmd = ({0}.register({1}), {2})"
+                fdef = fdef.format(target, original_cb, tk_args)
+                cbname = '_validatecmd'
+                lines.append(fdef)
+        line = '{0}.configure({1}={2})'.format(target, cmd_pname, cbname)
+        lines.append(line)
+        return lines
+
     def code_connect_bindings(self):
         lines = []
         target = self.code_identifier()
@@ -543,6 +617,7 @@ class BuilderObject(object):
 #
 class EntryBaseBO(BuilderObject):
     """Base class for tk.Entry and ttk.Entry builder objects"""
+
     def _set_property(self, target_widget, pname, value):
         if pname == 'text':
             wstate = str(target_widget['state'])
@@ -553,8 +628,9 @@ class EntryBaseBO(BuilderObject):
             target_widget.insert('0', value)
             target_widget['state'] = wstate
         else:
-            super(EntryBaseBO, self)._set_property(target_widget, pname, value)
-    
+            super(EntryBaseBO, self)._set_property(
+                target_widget, pname, value)
+
     #
     # Code generation methods
     #
@@ -564,13 +640,14 @@ class EntryBaseBO(BuilderObject):
                 "_text_ = '''{0}'''".format(value),
                 "{0}.delete('0', 'end')".format(targetid),
                 "{0}.insert('0', _text_)".format(targetid),
-                ]
+            ]
             if 'state' in self.wmeta.properties:
                 state_value = self.wmeta.properties['state']
                 if state_value != 'normal':
                     line = "{0}['state'] = 'normal'".format(targetid)
                     lines.insert(1, line)
-                    line = "{0}['state'] = '{1}'".format(targetid, state_value)
+                    line = "{0}['state'] = '{1}'".format(
+                        targetid, state_value)
                     lines.append(line)
             code_bag[pname] = lines
         else:
@@ -616,20 +693,20 @@ class PanedWindowPaneBO(BuilderObject):
 
     def add_child(self, bobject):
         self.widget.add(bobject.widget, **self.wmeta.properties)
-    
+
     #
     # code generation functions
     #
     def code_realize(self, boparent, code_identifier=None):
         self._code_identifier = boparent.code_child_master()
         return tuple()
-    
+
     def code_configure(self, targetid=None):
         return tuple()
-    
-    def code_layout(self, targetid=None):
+
+    def code_layout(self, targetid=None, parentid=None):
         return tuple()
-    
+
     def code_child_add(self, childid):
         config = []
         masterid = self.code_child_master()
